from .CUTE import *
from .functions import *
from .diagonal_functions import *
from .quadratic_functions import *
from .full_hessian_functions import *
from .DIXMAAN import DIXMAAN

test_function_register = dict()
test_function_register["ExtendedFreudenstein&Roth"] = ext_freudenstein_and_roth
test_function_register["ExtendedTrigonometric"] = ext_trigonometric
test_function_register["ExtendedRosenbrock"] = ext_rosenbrock
test_function_register["GeneralizedRosenbrock"] = gen_rosenbrock
test_function_register["ExtendedWhiteHolst"] = ext_white_and_holst
test_function_register["ExtendedBeale"] = ext_beale
test_function_register["ExtendedPenalty"] = ext_penalty
test_function_register["Raydan1"] = raydan_1
test_function_register["Raydan2"] = raydan_2
test_function_register["Hager"] = hager
test_function_register["ExtendedThreeExponentialTerms"] = ext_three_exponential_terms
test_function_register["ExtendedHimmelblau"] = ext_himmelblau
test_function_register["GeneralizedWhiteHolst"] = gen_white_and_holst
test_function_register["GeneralizedPSC1"] = gen_PCS1
test_function_register["ExtendedPSC1"] = ext_PSC1
test_function_register["ExtendedPowell"] = ext_powell
test_function_register["ExtendedMaratos"] = ext_maratos
test_function_register["ExtendedCliff"] = ext_cliff
test_function_register["ExtendedWood"] = ext_wood
test_function_register["ExtendedHiebert"] = ext_hiebert
test_function_register["Staircase1"] = staircase_1
test_function_register["Staircase2"] = staircase_2
test_function_register["Diagonal1"] = diagonal_1
test_function_register["Diagonal2"] = diagonal_2
test_function_register["Diagonal3"] = diagonal_3
test_function_register["Diagonal4"] = diagonal_4
test_function_register["Diagonal5"] = diagonal_5
test_function_register["Diagonal6"] = diagonal_6
test_function_register["Diagonal7"] = diagonal_7
test_function_register["Diagonal8"] = diagonal_8
test_function_register["Diagonal9"] = diagonal_9
test_function_register["ExtendedBlockDiagonal"] = ext_block_diagonal
test_function_register["ExtendedTridiagonal1"] = ext_tridiagonal_1
test_function_register["ExtendedTridiagonal2"] = ext_tridiagonal_2
test_function_register["GeneralizedTridiagonal1"] = gen_tridiagonal_1
test_function_register["GeneralizedTridiagonal2"] = gen_tridiagonal_2
test_function_register["BroydenTridiagonal"] = broyden_tridiagonal
test_function_register["PerturbedTridiagonalQuadratic"] = perturbed_tridiagonal_quadratic
test_function_register["PerturbedQuadratic"] = perturbed_quadratic
test_function_register["QuadraticQF1"] = quadratic_1
test_function_register["QuadraticQF2"] = quadratic_2
test_function_register["ExtendedQuadraticPenalty1"] = ext_quadratic_penalty_1
test_function_register["ExtendedQuadraticPenalty2"] = ext_quadratic_penalty_2
test_function_register["ExtendedQuadraticExponential"] = ext_quadratic_exponential
test_function_register["PartialPerturbedQuadratic"] = partial_perturbed_quadratic
test_function_register["AlmostPerturbedQuadratic"] = almost_perturbed_quadratic
test_function_register["Generalized Quadratic"] = gen_quadratic
test_function_register["FullHessian1"] = full_hessian_1
test_function_register["FullHessian2"] = full_hessian_2
test_function_register["FullHessian3"] = full_hessian_3
test_function_register["FLETCBV3"] = FLETCBV3
test_function_register["FLETCHCR"] = FLETCHCR
test_function_register["BDQRTIC"] = BDQRTIC
test_function_register["TRIDIA"] = TRIDIA
test_function_register["ARGLINB"] = ARGLINB
test_function_register["ARWHEAD"] = ARWHEAD
test_function_register["NONDIA"] = NONDIA
test_function_register["NONDQUAR"] = NONDQUAR
test_function_register["DQDRTIC"] = DQDRTIC
test_function_register["EG2"] = EG2
test_function_register["CURLY20"] = CURLY20
test_function_register["DIXMAANA"] = DIXMAAN('A')
test_function_register["DIXMAANB"] = DIXMAAN('B')
test_function_register["DIXMAANC"] = DIXMAAN('C')
test_function_register["DIXMAAND"] = DIXMAAN('D')
test_function_register["DIXMAANE"] = DIXMAAN('E')
test_function_register["DIXMAANF"] = DIXMAAN('F')
test_function_register["DIXMAANG"] = DIXMAAN('G')
test_function_register["DIXMAANH"] = DIXMAAN('H')
test_function_register["DIXMAANI"] = DIXMAAN('I')
test_function_register["DIXMAANJ"] = DIXMAAN('J')
test_function_register["DIXMAANK"] = DIXMAAN('K')
test_function_register["DIXMAANL"] = DIXMAAN('L')
test_function_register["LIARWHD1"] = LIARWHD_1
test_function_register["POWER"] = POWER
test_function_register["ENGVAL1"] = ENGVAL1
test_function_register["CRAGGLVY"] = CRAGGLVY
test_function_register["EDENSCH"] = EDENSCH
test_function_register["INDEF"] = INDEF
test_function_register["CUBE"] = CUBE
test_function_register["EXPLIN1"] = EXPLIN1
test_function_register["EXPLIN2"] = EXPLIN2
test_function_register["ARGLINC"] = ARGLINC
test_function_register["BDEXP"] = BDEXP
test_function_register["HARKERP2"] = HARKERP2
test_function_register["GENHUMPS"] = GENHUMPS
test_function_register["MCCORMCK"] = MCCORMCK
test_function_register["NONSCOMP"] = NONSCOMP
test_function_register["VARDIM"] = VARDIM
test_function_register["QUARTC"] = QUARTC
test_function_register["SINQUAD"] = SINQUAD
test_function_register["ExtendedDENSCHNB"] = ext_DENSCHNB
test_function_register["ExtendedDENSCHNF"] = ext_DENSCHNF
test_function_register["LIARWHD2"] = LIARWHD_2
test_function_register["DIXON3DQ"] = DIXON3DQ
test_function_register["COSINE"] = COSINE
test_function_register["SINE"] = SINE
test_function_register["BIGGSB1"] = BIGGSB1
test_function_register["SINCOS"] = SINCOS
test_function_register["HIMMELBG"] = HIMMELBG
test_function_register["HIMMELH"] = HIMMELH


class TestFunctionsError(Exception):
    def __init__(self, data, message):
        super().__init__()
        self.data = data
        self.message = message

    def __repr__(self):
        super().__str__()

    def __str__(self):
        return str(self.message) + ': ' + str(self.data)


class DimensionError(TestFunctionsError):
    def __init__(self, func_name, number, dimension, min_dim=2):
        msg = "The %s dimension must be multiple of %i and be greater or equal to %i. Dimension"
        msg = msg % (func_name, number, min_dim)
        super().__init__(dimension, msg)


class MinDimensionError(TestFunctionsError):
    def __init__(self, min_dimension, dimesion):
        msg = "The dimesion must be at least %. Dimension" % min_dimension
        super().__init__(dimesion, msg)


from .test_function import *
from .support_funcs import *
from .exceptions import *


def ext_freudenstein_and_roth(n):
    name = "Extended Freudenstein & Roth function"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm_1 = lambda i: (-13 + xi(2*i-1) + ((5 - xi(2*i))*xi(2*i) - 2)*xi(2*i))**2
    sm_2 = lambda i: (-29 + xi(2*i-1) + ((xi(2*i) + 1)*xi(2*i) - 14)*xi(2*i))**2
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = construct_x0([[0.5], [-2.0]], n)
    return create_test_function(name, n, sm, x0)


def ext_trigonometric(n):
    name = "Extended Trigonometric function"
    sm_1 = lambda i: n - sum((sp.cos(xi(j)) for j in range(1, n + 1)))
    sm_2 = lambda i: i*(1 - sp.cos(xi(i))) - sp.sin(xi(i))
    sm = lambda i: (sm_1(i) + sm_2(i))**2
    x0 = np.ones((n, 1)) * 0.2
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def ext_rosenbrock(n):
    name = "Extended Rosenbrock function"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm = lambda i: 100 * (xi(2*i) - xi(2*i-1) ** 2) ** 2 + (1 - xi(2*i - 1)) ** 2
    x0 = construct_x0([[1], [-1.2]], n)
    return create_test_function(name, n, sm, x0)


def gen_rosenbrock(n):
    name = "Generalized Rosenbrock function"
    sm = lambda i: 100*(xi(i+1) - xi(i)**2)**2 + (1 - xi(i))**2
    x0 = construct_x0([[1], [-1.2]], n)
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def ext_white_and_holst(n):
    name = "Extended White & Holst function"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm_1 = lambda i: 100 * (xi(2 * i) - xi(2 * i - 1) ** 3) ** 2
    sm_2 = lambda i: (1 - xi(2 * i - 1)) ** 2
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = construct_x0([[1], [-1.2]], n)
    return create_test_function(name, n, sm, x0)


def ext_beale(n):
    name = "Extended Beale function"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm_1 = lambda i: (1.5 - xi(2*i-1)*(1-xi(2*i)))**2
    sm_2 = lambda i: (2.25 - xi(2*i-1)*(1-xi(2*i)**2))**2
    sm_3 = lambda i: (2.625 - xi(2 * i - 1) * (1 - xi(2 * i)**3)) ** 2
    sm = lambda i: sm_1(i) + sm_2(i) + sm_3(i)
    x0 = construct_x0([[1], [0.8]], n)
    return create_test_function(name, n, sm, x0)


def ext_penalty(n):
    name = "Extended Penalty function"
    sm_1 = lambda i: (xi(i) - 1)**2
    sm_2 = lambda: (sum((xi(j)**2 for j in range(1, n + 1))) - 0.25)**2
    sm = lambda i: sm_1(i) + sm_2
    x0 = np.arange(1, n + 1).reshape((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def raydan_1(n):
    name = "Raydan 1 function"
    sm = lambda i: i * (sp.exp(xi(i)) - xi(i)) / 10
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def raydan_2(n):
    name = "Raydan 2 function"
    sm = lambda i: sp.exp(xi(i)) - xi(i)
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def hager(n):
    name = "Hager function"
    sm = lambda i: sp.exp(xi(i)) - sp.sqrt(i) * xi(i)
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def ext_three_exponential_terms(n):
    name = "Extended TET function : (Three exponential terms)"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm_1 = lambda i: sp.exp(xi(2*i-1) + 3*xi(2*i) - 0.1)
    sm_2 = lambda i: sp.exp(xi(2*i-1) - 3*xi(2*i) - 0.1)
    sm_3 = lambda i: sp.exp(-xi(2*i-1) - 0.1)
    sm = lambda i: sm_1(i) + sm_2(i) + sm_3(i)
    x0 = np.ones((n, 1)) * 0.1
    return create_test_function(name, n, sm, x0)


def ext_himmelblau(n):
    name = "Extended Himmelblau function"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm_1 = lambda i: (xi(2*i-1)**2 + xi(2*i) - 11)**2
    sm_2 = lambda i: (xi(2*i-1) + xi(2*i)**2 - 7)**2
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0)


def gen_white_and_holst(n):
    name = "Generalized White & Holst function"
    sm = lambda i: 100 * (xi(i+1) - xi(i)**3)**2 + (1 - xi(i))**2
    x0 = construct_x0([[1], [-1.2]], n)
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def gen_PCS1(n):
    name = "Generalized PSC1 function"
    sm_1 = lambda i: (xi(i)**2 + xi(i+1)**2 + xi(i)*xi(i+1))**2
    sm_2 = lambda i: sp.sin(xi(i))**2 + sp.cos(xi(i))**2
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = construct_x0([[3.0], [0.1]], n)
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def ext_PSC1(n):
    name = "Extended PSC1 function"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm_1 = lambda i: (xi(2*i-1)**2 + xi(2*i)**2 + xi(2*i-1)*xi(2*i))**2
    sm_2 = lambda i: sp.sin(xi(2*i-1))**2 + sp.cos(xi(2*i))**2
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = construct_x0([[3.0], [0.1]], n)
    return create_test_function(name, n, sm, x0)


def ext_powell(n):
    name = "Extended Powell function"
    if n % 4:
        raise DimensionError(name, 4, n)
    sm_1 = lambda i: (xi(4*i-3) + 10*xi(4*i-2))**2 + 5*(xi(4*i-1) - xi(4*i))**2
    sm_2 = lambda i: (xi(4*i-2) - 2*xi(4*i-1))**4 + 10*(xi(4*i - 3) - xi(4*i))**4
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = construct_x0([[-3.0], [-1.0], [0.0], [1.0]], n)
    return create_test_function(name, n, sm, x0,
                                range_func=default_range, limits=(1, n // 4 + 1), min_dimesion=4)


def ext_maratos(n):
    name = "Extended Maratos function"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm = lambda i: xi(2*i-1) + 100*(xi(2*i-1)**2 + xi(2*i)**2 - 1)**2
    x0 = construct_x0([[1.1], [0.1]], n)
    return create_test_function(name, n, sm, x0)


def ext_cliff(n):
    name = "Extended Cliff function"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm_1 = lambda i: ((xi(2*i-1) - 3) / 100)**2 - (xi(2*i-1) - xi(2*i))
    sm_2 = lambda i: sp.exp(20*(xi(2*i-1) - xi(2*i)))
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = construct_x0([[0.0], [-1.0]], n)
    return create_test_function(name, n, sm, x0)


def ext_wood(n):
    name = "Extended Wood function"
    if n % 4:
        raise DimensionError(name, 4, n, 4)
    sm_1 = lambda i: 100 * (xi(4*i-3)**2 - xi(4*i-2))**2 + (xi(4*i-3) - 1)**2
    sm_2 = lambda i: 90 * (xi(4*i-2)**2 + xi(4*i))**2 + (1 - xi(4*i-1))**2
    sm_3 = lambda i: 10.1 * ((xi(4*i-2) - 1)**2 + (xi(4*i) - 1)**2)
    sm_4 = lambda i: 19.8 * (xi(4*i-2) - 1)*(xi(4*i) - 1)
    sm = lambda i: sm_1(i) + sm_2(i) + sm_3(i) + sm_4(i)
    x0 = construct_x0([[-3.0], [-1.0], [-3.0], [-1.0]], n)
    return create_test_function(name, n, sm, x0,
                                range_func=default_range, limits=(1, n // 4 + 1), min_dimesion=4)


def ext_hiebert(n):
    name = "Extended Hiebert function"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm = lambda i: (xi(2*i-1) - 10)**2 + (xi(2*i-1)*xi(2*i) - 50000)**2
    x0 = np.zeros((n, 1))
    return create_test_function(name, n, sm, x0)


def staircase_1(n):
    name = "Staircase 1 function"
    sm = lambda i: sum((xi(j) for j in range(1, i + 1)))**2
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def staircase_2(n):
    name = "Staircase 2 function"
    sm = lambda i: (sum((xi(j) for j in range(1, i + 1))) - i)**2
    x0 = np.zeros((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_1)




from .test_function import *
from .support_funcs import *
from .exceptions import *


def perturbed_quadratic(n):
    name = "Perturbed Quadratic function"
    f = lambda: 0.01 * sum([xi(i) for i in range(1, n + 1)]) ** 2
    sm = lambda i: i * xi(i) ** 2
    x0 = np.ones((n, 1)) * 0.5
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range_1)


def quadratic_1(n):
    name = "Quadratic QF1 function"
    sm = lambda i: 0.5 * (i * xi(i) ** 2 - xi(n))
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def quadratic_2(n):
    name = "Quadratic QF2 function"
    sm = lambda i: 0.5 * (i * (xi(i) ** 2 - 1) ** 2 - xi(n))
    x0 = np.ones((n, 1)) * 0.5
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def ext_quadratic_penalty_1(n):
    name = "Extended quadratic penalty QP1 function"
    f = lambda: (sum([xi(i) ** 2 for i in range(1, n + 1)]) - 0.5) ** 2
    sm = lambda i: (xi(i) ** 2 - 2) ** 2
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range_3)


def ext_quadratic_penalty_2(n):
    name = "Extended quadratic penalty QP2 function"
    f = lambda: (sum([xi(i) ** 2 for i in range(1, n + 1)]) - 100) ** 2
    sm = lambda i: (xi(i) ** 2 - sp.sin(xi(i))) ** 2
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range_3)


def ext_quadratic_exponential(n):
    name = "Extended quadratic exponential EP1 function"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm_1 = lambda i: (sp.exp(xi(2*i-1) - xi(2*i)) - 5) ** 2
    sm_2 = lambda i: ((xi(2*i-1) - xi(2*i))**2) * (xi(2*i-1) - xi(2*i) - 11)**2
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = np.ones((n, 1)) * 1.5
    return create_test_function(name, n, sm, x0)


def partial_perturbed_quadratic(n):
    name = "Partial Perturbed Quadratic function"
    f = lambda: xi(1) ** 2
    sm = lambda i: i * xi(i) ** 2 + 0.01 * sum([xi(j) for j in range(1, i + 1)])**2
    x0 = np.ones((n, 1)) * 0.5
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range_1)


def almost_perturbed_quadratic(n):
    name = "Almost Perturbed Quadratic function"
    sm = lambda i: i * xi(i) ** 2 + 0.01 * (xi(1) + xi(n)) ** 2
    x0 = np.ones((n, 1)) * 0.5
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def gen_quadratic(n):
    name = "Generalized Quadratic function"
    sm = lambda i: xi(i) ** 2 + (xi(i+1) + xi(i) ** 2) ** 2
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


import numpy as np


def default_range(start, stop):
    """[start, start + 1, ..., stop -1]"""
    return range(start, stop)


def default_range_1(n):
    """[1, 2, ..., n]"""
    return range(1, n + 1)


def default_range_2(n):
    """[1, 2, ..., n // 2]"""
    return range(1, n // 2 + 1)


def default_range_3(n):
    """[1, 2, ..., n - 1]"""
    return range(1, n)


def construct_x0(base, n):
    return np.array(base * (n // len(base)))



from .test_function import *
from .support_funcs import *
import sympy as sp
import numpy as np


def full_hessian_1(n):
    name = "Full Hessian FH1 function"
    f = lambda: (xi(1) - 3) ** 2
    sm = lambda i: (xi(1) - 3 - 2 * sum([xi(j) for j in range(1, i + 1)])**2)**2
    x0 = np.ones((n, 1)) * 0.01
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range, limits=(2, n + 1))


def full_hessian_2(n):
    name = "Full Hessian FH2 function"
    f = lambda: (xi(1) - 5) ** 2
    sm = lambda i: (sum([xi(j) for j in range(1, i + 1)]) - 1)**2
    x0 = np.ones((n, 1)) * 0.01
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range_1)


def full_hessian_3(n):
    name = "Full Hessian FH3 function"
    f = lambda: sum([xi(i) for i in range(1, n + 1)]) ** 2
    sm = lambda i: (xi(i) * sp.exp(xi(i)) - 2*xi(i) - xi(i)**2)
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range_1)


import sympy as sp
import numpy as np
import matplotlib.pyplot as plt
import mpl_toolkits.mplot3d as plt3d
from .support_funcs import default_range_2
from .exceptions import MinDimensionError


"""
Test function types are:
  1.Unimodal function.
  2.Functions with significant null-space effects.
  3.Essentially unimodal functions.
  4.Functions with a huge number of significant local optima.
  5.Functions with a small number of significant local optima
  6.Functions whose global structure provides no useful information about its optima.
"""
#TODO put each test function in it's type python file


class TestFunction:
    """
    TestFunction class describe a test functions for unconstrained optimization.
    It based on sympy function representation.
    So it consist of function variables, function form, default x0 and minimum dimension.
    You can lamdify
    """
    def __init__(self, name, function, x0, min_dimesion):
        self.variables = function.free_symbols
        self.dimension = len(self.variables)
        if self.dimension < min_dimesion:
            raise MinDimensionError(min_dimesion, self.dimension)
        self.name = name
        self.sympy_function = function
        self.x0 = x0

    def lambdify(self):
        f = sp.lambdify(self.variables, self.sympy_function)
        return lambda x: f(*x)

    def plot_surface_3d(self, x1, x2):
        if self.dimension == 2:
            fig = plt.figure(self.name)
            ax = fig.gca(projection='3d')
            X = x1
            Y = x2
            func = self.lambdify()
            X, Y = np.meshgrid(X, Y)
            Z = np.array([func((x, y)) for x, y in zip(X.ravel(), Y.ravel())]).reshape(X.shape)
            ax.plot_surface(X, Y, Z)
        else:
            print("The dimesion is too high for plotting")

    @property
    def get_name(self):
        return self.name

    @property
    def get_variables(self):
        return self.variables

    @property
    def get_function(self):
        return self.sympy_function

    @property
    def get_dimension(self):
        return self.dimension

    @property
    def get_x0(self):
        return self.x0

    def __str__(self):
        result = "Test function name: {self.name}\n" \
                 "Function dimension: {self.dimension}\n" \
                 "Function form: {self.sympy_function}\n" \
                 "Default x0: \n{self.x0}"
        return result.format(self=self)


def xi(i):
    return sp.Symbol('x%i' % i)


def summation(series_member, series_range):
    return sum([series_member(i) for i in series_range])


def create_test_function(name, n, series_member, x0,
                         first=None, range_func=default_range_2, limits=None,
                         min_dimesion=2):
    if limits:
        series_range = range_func(*limits)
    else:
        series_range = range_func(n)
    func = summation(series_member, series_range)
    if first:
        func += first()
    return TestFunction(name, func, x0, min_dimesion)







from .test_function import *
from .support_funcs import *
from .exceptions import *

def diagonal_1(n):
    name = "Diagonal 1 function"
    sm = lambda i: sp.exp(xi(i)) - i * xi(i)
    x0 = np.ones((n, 1)) * (1 / n)
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def diagonal_2(n):
    name = "Diagonal 2 function"
    sm = lambda i: sp.exp(xi(i)) - xi(i) / i
    x0 = (1 / np.arange(1, n + 1)).reshape((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def diagonal_3(n):
    name = "Diagonal 3 function"
    sm = lambda i: sp.exp(xi(i)) - i * sp.sin(xi(i))
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def diagonal_4(n):
    name = "Diagonal 4 function"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm = lambda i: 0.5 * (xi(2*i-1) + 100*xi(2*i))**2
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0)


def diagonal_5(n):
    name = "Diagonal 5 function"
    sm = lambda i: sp.log(sp.exp(xi(i)) + sp.exp(-xi(i)))
    x0 = np.ones((n, 1)) * 1.1
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def diagonal_6(n):
    name = "Diagonal 6 function"
    sm = lambda i: sp.exp(xi(i)) + (1 - xi(i))
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def diagonal_7(n):
    name = "Diagonal 7 function"
    sm = lambda i: sp.exp(xi(i)) - 2 * xi(i) - xi(i) ** 2
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def diagonal_8(n):
    name = "Diagonal 8 function"
    sm = lambda i: xi(i) * sp.exp(xi(i)) - 2 * xi(i) - xi(i) ** 2
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def diagonal_9(n):
    name = "Diagonal 9 function"
    sm = lambda i: (sp.exp(xi(i)) - i * xi(i)) + 10000 * xi(n) ** 2
    x0 = np.ones((n, 1)) * 1.5
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def ext_block_diagonal(n):
    name = "Extended BD1 function (Block Diagonal)"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm_1 = lambda i: (xi(2*i-1)**2 + xi(2*i)**2 - 2) ** 2
    sm_2 = lambda i: (sp.exp(xi(2*i-1)-1) - xi(2*i)) ** 2
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = np.ones((n, 1)) * 0.1
    return create_test_function(name, n, sm, x0)


def ext_tridiagonal_1(n):
    name = "Extended Tridiagonal 1 function"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm_1 = lambda i: (xi(2*i-1) + xi(2*i) - 3) ** 2
    sm_2 = lambda i: (xi(2*i-1) - xi(2*i) + 1) ** 4
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = np.ones((n, 1)) * 2.0
    return create_test_function(name, n, sm, x0)


def gen_tridiagonal_1(n):
    name = "Generalized Tridiagonal 1 function"
    sm_1 = lambda i: (xi(i) + xi(i+1) - 3) ** 2
    sm_2 = lambda i: (xi(i) - xi(i+1) + 1) ** 4
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = np.ones((n, 1)) * 2.0
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def gen_tridiagonal_2(n):
    name = "Generalized Tridiagonal 2 function"
    f = lambda: ((5 - 3*xi(1) - xi(2)**2) * xi(1) - 3*xi(2) + 1) ** 2
    sm_1 = lambda i: ((5 - 3*xi(i) - xi(i)**2) * xi(1) - xi(i-1) - 3*xi(i+1) + 1) ** 2
    sm_2 = lambda: ((5 - 3*xi(n) - xi(n)**2) * xi(n) - xi(n-1) + 1) ** 2
    sm = lambda i: sm_1(i) + sm_2()
    x0 = np.ones((n, 1)) * -1.0
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range, limits=(2, n))


def ext_tridiagonal_2(n):
    name = "Extended Tridiagonal 2 function"
    sm_1 = lambda i: (xi(i) * xi(i+1) - 1) ** 2
    sm_2 = lambda i: 0.1 * (xi(i) + 1) * (xi(i+1) + 1)
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def broyden_tridiagonal(n):
    name = "Broyden Tridiagonal function"
    f = lambda: (3 * xi(1) - 2 * xi(1) ** 2) ** 2
    sm_1 = lambda i: (3*xi(i) - 2*xi(i)**2 - xi(i-1) - 2*xi(i+1) + 1) ** 2
    sm_2 = lambda: (3*xi(n) - 2*xi(n)**2 - xi(n-1) + 1) ** 2
    sm = lambda i: sm_1(i) + sm_2()
    x0 = np.ones((n, 1)) * -1.0
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range, limits=(2, n))


def perturbed_tridiagonal_quadratic(n):
    name = "Perturbed Tridiagonal Quadratic function"
    f = lambda: xi(1) ** 2
    sm = lambda i: i * xi(i) ** 2 + (xi(i-1) + xi(i) + xi(i+1)) ** 2
    x0 = np.ones((n, 1)) * 0.5
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range, limits=(2, n))



from .test_function import *
from .support_funcs import *
from .exceptions import *


def FLETCBV3(n):
    name = "FLETCBV3 function (CUTE)"
    print(name)
    p, h = 1e-8, 1 / (n + 1)
    # move last n-member of second series to f and and create one series from 1 to n-1
    f = lambda: 0.5*p*(xi(1) + xi(n))**2 - ((h**2 + 2) * xi(n) - sp.cos(xi(n))) * p / h**2
    sm_1 = lambda i: 0.5*p*(xi(i) - xi(i+1))
    sm_2 = lambda i: -((h**2 + 2) * xi(i) - sp.cos(xi(i))) * p / h**2
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = (np.arange(1, n + 1) * h).reshape((n, 1))
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range_3)


def FLETCHCR(n):
    name = "FLETCHCR function (CUTE)"
    sm = lambda i: 100 * (xi(i+1) - xi(i) + 1 - xi(i)**2)**2
    x0 = np.zeros((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def BDQRTIC(n):
    name = "BDQRTIC function (CUTE)"
    sm_1 = lambda i: (-4*xi(i) + 3)**2
    sm_2 = lambda i: (xi(i) + 2*xi(i+1)**2 + 3*xi(i+2)**2 + 4*xi(i+3) + 5*xi(n)**2) ** 2
    sm = lambda i:  sm_1(i) + sm_2(i)
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range, limits=(1, n - 3))


def TRIDIA(n):
    name = "TRIDIA function (CUTE)"
    alpha, beta, gamma, sigma = 2, 1, 1, 1
    f = lambda: gamma * (sigma * xi(1) - 1) ** 2
    sm = lambda i:  i * (alpha * xi(i) - beta * xi(i-1)) ** 2
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, first=f,
                                range_func=default_range, limits=(2, n + 1))


def ARGLINB(n):
    name = "ARGLINB function (CUTE)"
    sm = lambda i: sum([i*j*xi(j) - 1 for j in range(1, n + 1)]) ** 2
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def ARWHEAD(n):
    name = "ARWHEAD function (CUTE)"
    sm = lambda i: (-4*xi(i) + 3)**2 + (xi(i)**2 + xi(n)**2) ** 2
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def NONDIA(n):
    name = "NONDIA function (CUTE)"
    f = lambda: (xi(1) - 1) ** 2
    sm = lambda i: 100 * (xi(i) - xi(i-1)**2)**2
    x0 = np.ones((n, 1)) * -1.0
    return create_test_function(name, n, sm, x0,
                                first=f, range_func=default_range, limits=(2, n + 1))


def NONDQUAR(n):
    name = "NONDQUAR function (CUTE)"
    f = lambda: (xi(1) - xi(2)) ** 2
    sm_1 = lambda i: (xi(i) + xi(i+1) + xi(n)) ** 4
    sm_2 = lambda: (xi(n-1) + xi(n)) ** 2
    sm = lambda i: sm_1(i) + sm_2()
    x0 = construct_x0([[1.0], [-1.0]], n)
    return create_test_function(name, n, sm, x0,
                                first=f, range_func=default_range, limits=(2, n - 1))


def DQDRTIC(n):
    name = "DQDRTIC function (CUTE)"
    c, d = 100, 100
    sm = lambda i: xi(i) + c*xi(i+1)**2 + d*xi(i+2)**2
    x0 = np.ones((n, 1)) * 3.0
    return create_test_function(name, n, sm, x0,
                                range_func=default_range, limits=(2, n - 1), min_dimesion=3)


def EG2(n):
    name = "EG2 function (CUTE)"
    sm = lambda i: sp.sin(xi(1) + xi(i)**2 - 1) + 0.5 * sp.sin(xi(n)**2)
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def CURLY20(n):
    def q(i, k=20):
        if i <= n - k:
            return sum([xi(i) for i in range(1, i + k + 1)])
        else:
            return sum([xi(i) for i in range(1, n + 1)])
    name = "CURLY20 function (CUTE)"
    sm = lambda i: q(i)**2 - 20*q(i)**2 - 0.1*q(i)
    x0 = np.ones((n, 1)) * 0.001 / (n + 1)
    return create_test_function(name, n, sm, x0, range_func=default_range_1)



def LIARWHD_1(n):
    name = "LIARWHD1 function (CUTE)"
    sm = lambda i: 4*(-xi(1) + xi(i)**2)**2 + (xi(i) - 1)**2
    x0 = np.ones((n, 1)) * 4.0
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def POWER(n):
    name = "POWER function (CUTE)"
    sm = lambda i: (i * xi(i)) ** 2
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def ENGVAL1(n):
    name = "ENGVAL1 function (CUTE)"
    sm = lambda i: (xi(i)**2 + xi(i+1)**2)**2 + (-4*xi(i) + 3)
    x0 = np.ones((n, 1)) * 2.0
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def CRAGGLVY(n):
    name = "CRAGGLVY function (CUTE)"
    if n % 2 or n < 4:
        raise DimensionError(name, 2, n, 4)
    sm_1 = lambda i: (sp.exp(xi(2*i-1) - xi(2*i)))**4 + 100*(xi(2*i) - xi(2*i-1))**6
    sm_2 = lambda i: (sp.tan(xi(2*i+1)+xi(2*i+2)) + xi(2*i+1) - xi(2*i+2))**4
    sm_3 = lambda i: xi(2*i-1)**8 + (xi(2*i+2)-1)**2
    sm = lambda i: sm_1(i) + sm_2(i) + sm_3(i)
    x0 = np.ones((n, 1)) * 2.0
    x0[0][0] = 1.0
    return create_test_function(name, n, sm, x0, range_func=default_range, limits=(1, n // 2))


def EDENSCH(n):
    name = "EDENSCH function (CUTE)"
    f = lambda: 16
    sm = lambda i: (xi(i) - 2)**4 + (xi(i)*xi(i+1) - 2*xi(i+1))**2 +(xi(i+1) + 1)**2
    x0 = np.zeros((n, 1))
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range_3)


def INDEF(n):
    name = "INDEF function (CUTE)"
    f = lambda: xi(1) + xi(n)
    sm = lambda i: xi(i) + 0.5 * sp.cos(2*xi(i) - xi(n) - xi(1))
    x0 = (np.arange(1, n + 1) * 1 / (n + 1)).reshape((n, 1))
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range, limits=(2, n))


def CUBE(n):
    name = "CUBE function (CUTE)"
    f = lambda: (xi(1) - 1)**2
    sm = lambda i: 100*(xi(i) - xi(i-1)**3)**2
    x0 = construct_x0([[-1.2], [1.0]], n)
    return create_test_function(name, n, sm, x0,
                                first=f, range_func=default_range, limits=(2, n + 1))


def EXPLIN1(n):
    name = "EXPLIN1 function (CUTE)"
    f = lambda: -10 * n * xi(n)
    sm = lambda i: sp.exp(0.1*xi(i)*xi(i+1)) - 10*(i*xi(i))
    x0 = np.zeros((n, 1))
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range_3)


def EXPLIN2(n):
    name = "EXPLIN2 function (CUTE)"
    f = lambda: -10 * n * xi(n)
    sm = lambda i: sp.exp(i*xi(i)*xi(i+1) / (10*n)) - 10*(i*xi(i))
    x0 = np.zeros((n, 1))
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range_3)


def ARGLINC(n):
    name = "ARGLINC function (CUTE)"
    f = lambda: 2
    sm = lambda i: sum([j*xi(j)*(i-1) - 1 for j in range(2, n)])**2
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0,
                                first=f, range_func=default_range, limits=(2, n))


def BDEXP(n):
    name = "BDEXP function (CUTE)"
    sm = lambda i: (xi(i) + xi(i+1)) * sp.exp(-xi(i+2)*(xi(i) + xi(i+1)))
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range, limits=(1, n - 1))


def HARKERP2(n):
    name = "HARKERP2 function (CUTE)"
    f1 = lambda j: sum([xi(j) for j in range(j, n + 1)]) ** 2
    f2 = lambda: sum([f1(j) for j in range(2, n + 1)])
    f = lambda: f1(1) + 2 * f2()
    sm = lambda i: -(xi(i) + 0.5*xi(i)**2)**2
    x0 = np.arange(1, n + 1).reshape((n, 1))
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range_1)


def GENHUMPS(n):
    name = "GENHUMPS function (CUTE)"
    sm_1 = lambda i: (sp.sin(2*xi(i))**2) * sp.sin(2*xi(i+1))**2
    sm_2 = lambda i: 0.05*(xi(i)**2 + xi(i+1)**2)
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = np.ones((n, 1)) * -506.2
    x0[0][0] = 506.0
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def MCCORMCK(n):
    name = "MCCORMCK function (CUTE)"
    sm_1 = lambda i: -1.5*xi(i) + 2.5*xi(i+1) + 1
    sm_2 = lambda i: (xi(i) - xi(i+1))**2 + sp.sin(xi(i) + xi(i+1))
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def NONSCOMP(n):
    name = "NONSCOMP function (CUTE)"
    f = lambda: (xi(1) - 1) ** 2
    sm = lambda i: 4 * (xi(i) - xi(i-1)**2)**2
    x0 = np.ones((n, 1)) * 3.0
    return create_test_function(name, n, sm, x0,
                                first=f, range_func=default_range, limits=(2, n + 1))


def VARDIM(n):
    name = "VARDIM function (CUTE)"
    sm_f = lambda i: i* xi(i) - n * (n + 1) / 2
    ff = lambda: sum([sm_f(j) for j in range(1, n + 1)])
    f = lambda: ff() ** 2 + ff() ** 4
    sm = lambda i: (xi(i) - 1) ** 2
    x0 = (np.ones(n) - np.arange(1, n + 1) / n).reshape((n, 1))
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range_1)


def QUARTC(n):
    name = "QUARTC function (CUTE)"
    sm = lambda i: (xi(i) - 1) ** 4
    x0 = np.ones((n, 1)) * 2.0
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def SINQUAD(n):
    name = "SINQUAD function (CUTE)"
    f = lambda: (xi(1) - 1) ** 4
    sm_1 = lambda i: (sp.sin(xi(i) - xi(n)) - xi(1) + xi(i)**2)**2
    sm_2 = lambda: (xi(n)**2 - xi(1)**2) ** 2
    sm = lambda i: sm_1(i) + sm_2()
    x0 = np.ones((n, 1)) * 0.1
    return create_test_function(name, n, sm, x0,
                                first=f, range_func=default_range, limits=(2, n))


def ext_DENSCHNB(n):
    name = "Extended DENSCHNB function (CUTE)"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm_1 = lambda i: (xi(2*i-1) - 2)**2 +(xi(2*i-1) - 2) * xi(2*i) ** 2
    sm_2 = lambda i: (xi(2*i) + 1) ** 2
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0)


def ext_DENSCHNF(n):
    name = "Extended DENSCHNF function (CUTE)"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm_1 = lambda i: (2*(xi(2*i-1) + xi(2*i))**2 + (xi(2*i-1) - xi(2*i))**2 - 8) ** 2
    sm_2 = lambda i: (5*xi(2*i-1)**2 + (xi(2*i) - 3)**2 - 9) ** 2
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = construct_x0([[0.0], [2.0]], n)
    return create_test_function(name, n, sm, x0)


def LIARWHD_2(n):
    name = "LIARWHD2 function (CUTE)"
    sm = lambda i: 4*(xi(i)**2 - xi(i)) ** 2 + (xi(i) - 1) ** 2
    x0 = np.ones((n, 1)) * 4.0
    return create_test_function(name, n, sm, x0, range_func=default_range_1)


def DIXON3DQ(n):
    name = "DIXON3DQ function (CUTE)"
    f = lambda: (xi(1) - 1) ** 2
    sm = lambda i: (xi(i) - xi(i+1)) ** 2 + (xi(n) - 1) ** 2
    x0 = np.ones((n, 1)) * -1.0
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range_3)


def COSINE(n):
    name = "COSINE function (CUTE)"
    sm = lambda i: sp.cos(-0.5*xi(i+1) + xi(i)**2)
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def SINE(n):
    name = "SINE function (CUTE)"
    sm = lambda i: sp.sin(-0.5*xi(i+1) + xi(i)**2)
    x0 = np.ones((n, 1))
    return create_test_function(name, n, sm, x0, range_func=default_range_3)


def BIGGSB1(n):
    name = "BIGGSB1 function (CUTE)"
    f = lambda: (xi(1) - 1) ** 2
    sm = lambda i: (xi(i+1) - xi(i)) ** 2 + (1 - xi(n)) ** 2
    x0 = np.zeros((n, 1))
    return create_test_function(name, n, sm, x0, first=f, range_func=default_range_3)


def SINCOS(n):
    name = "SINCOS function (CUTE)"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm_1 = lambda i: (xi(2*i-1)**2 + xi(2*i)**2 + xi(2*i-1)*xi(2*i)) ** 2
    sm_2 = lambda i: sp.sin(xi(2*i-1)) ** 2 + sp.cos(xi(2*i)) ** 2
    sm = lambda i: sm_1(i) + sm_2(i)
    x0 = construct_x0([[3.0], [0.1]], n)
    return create_test_function(name, n, sm, x0)


def HIMMELBG(n):
    name = "HIMMELBG function (CUTE)"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm = lambda i: (2*xi(2*i-1)**2 + 3*xi(2*i)**2) * sp.exp(-xi(2*i-1) - xi(2*i))
    x0 = np.ones((n, 1)) * 1.5
    return create_test_function(name, n, sm, x0)


def HIMMELH(n):
    name = "HIMMELH function (CUTE)"
    if n % 2:
        raise DimensionError(name, 2, n)
    sm = lambda i: -3*xi(2*i-1) - 2*xi(2*i) + 2 + xi(2*i-1) ** 2 + xi(2*i) ** 2
    x0 = np.ones((n, 1)) * 1.5
    return create_test_function(name, n, sm, x0)


from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()
print(find_packages('UO', exclude=('setup.py')))
setup(
    name='UO-test-functions',
    version='1.0.3',
    description='An Unconstrained Optimization Test Functions Collection',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/SupremaLex/UO-test-funcs/',
    author='George Lutsenko',
    author_email='georglutsenko@gmail.com',
	
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
	'Intended Audience :: Education',
        'Topic :: Scientific/Engineering :: Mathematics',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    keywords='minimization unconstrained-optimization test-functions optimization',
    packages=['UO.test_functions'],#find_packages('UO',exclude=('setup.py')),
    package_data={'UO': ['ChangeLog'],},
    install_requires=['sympy', 'numpy', 'matplotlib'],
    project_urls={
        'Bug Reports': 'https://github.com/SupremaLex/UO-test-funcs/issues',
        'Source': 'https://github.com/SupremaLex/UO-test-funcs/',
    },
)





from .test_functions_register import test_function_register


class TestFunctionFactory:
    @staticmethod
    def create(function_name, function_dimesion):
        return test_function_register[function_name](function_dimesion)

    @property
    def get_all_functions_names(self):
        return list(test_function_register.keys())